@@ -1,12 +1,12 @@
 import json
+import itertools
 
 from datetime import datetime
 from itertools import chain
 
-from py42.response import Py42Response
-from py42.sdk.queries.query_filter import FilterGroup
+from py42.sdk.queries.query_filter import FilterGroup, QueryFilter
 from py42.sdk.queries.fileevents.file_event_query import FileEventQuery
-from py42.sdk.queries.fileevents.filters import InsertionTimestamp
+from py42.sdk.queries.fileevents.filters import InsertionTimestamp, EventTimestamp
 from py42.sdk.queries.alerts.alert_query import AlertQuery
 from py42.sdk.queries.alerts.filters.alert_filter import DateObserved
 
@@ -20,6 +20,13 @@ FILTER_ERROR = u"{} filter can't be used with {} when a cursor checkpoint exists
 class BaseExtractor(object):
     _max_page_size = 10000
 
+    use_or_query = False
+    # exclude timestamp filters by default from "OR" queries
+    # if other filters need to be exempt when building a query, append them to this list
+    # can either be a `QueryFilter` subclass, or a composed `FilterGroup` if more precision on
+    # is needed for which filters should be "AND"ed
+    or_query_exempt_filters = [InsertionTimestamp, EventTimestamp, DateObserved]
+
     def __init__(self, key, search_function, handlers, timestamp_filter, query_class):
         self._key = key
         self._search = search_function
@@ -95,12 +102,40 @@ class BaseExtractor(object):
         filter_groups.append(timestamp_filter)
 
     def _create_query_from_filters(self, filter_groups):
+        if self.use_or_query:
+            filter_groups = self._convert_to_or_query(filter_groups)
         query = self._query_class(*filter_groups)
         query.sort_direction = u"asc"
         query.sort_key = self._sort_key
         query.page_size = self._max_page_size
         return query
 
+    def _convert_to_or_query(self, filter_groups):
+        and_group = FilterGroup([], "AND")
+        or_group = FilterGroup([], "OR")
+        filters = itertools.chain.from_iterable([f.filter_list for f in filter_groups])
+        for _filter in filters:
+            if self._is_exempt_filter(_filter):
+                and_group.filter_list.append(_filter)
+            else:
+                or_group.filter_list.append(_filter)
+
+        if and_group.filter_list:
+            return [and_group, or_group]
+        else:
+            return [or_group]
+
+    def _is_exempt_filter(self, f):
+        for exempt in self.or_query_exempt_filters:
+            if isinstance(exempt, FilterGroup):
+                if f in exempt:
+                    return True
+                else:
+                    continue
+            elif f.term == exempt._term:
+                return True
+        return False
+
     def _get_results_from_response(self, response):
         return response[self._key]
 
